/*
 [The "BSD licence"]
 Copyright (c) 2005 Jean Bovet, http://www.cs.usfca.edu/~jbovet/
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
														   NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
														   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "Ethernal.h"
#import "PriviledgedSocketCreator.h"
#import "ITMArrayOfObjectsTransformer.h"

#define MORE_UNIX_WRITE_CHECK_SIGPIPE 0

@implementation Ethernal

static Ethernal *_shared = NULL;

+ (void)initialize
{
	NSValueTransformer *transformer = [[[ITMArrayOfObjectsTransformer alloc] init] autorelease];
	[NSValueTransformer setValueTransformer:transformer forName:@"ITMArrayOfObjectsTransformer"];
}

+ (Ethernal*)shared
{
	return _shared;
}

- (id)init
{
	if(self = [super init]) {
		_shared = self;
		mSocketDictionary = [[NSMutableDictionary alloc] init];
	}
	return self;
}

- (void)dealloc
{
	[mSocketDictionary release];
	[super dealloc];
}

- (void)applicationDidFinishLaunching:(NSNotification *)notif
{
	if(![[[NSUserDefaults standardUserDefaults] objectForKey:@"donated"] boolValue]) {
		NSAlert *alert = [NSAlert alertWithMessageText:@"Donate to support free software development!" 
										 defaultButton:@"I have donated!"
									   alternateButton:@"Donate..."
										   otherButton:@"Remind me later"
							 informativeTextWithFormat:@"Developing free software like Ethernal takes time and resources. Making a donation is an encouragement for me to continue work on these projects and to increase and improve the free software."];
		switch([alert runModal]) {
			case NSAlertDefaultReturn:
				[[NSAlert alertWithMessageText:@"Thank you very much!" 
								 defaultButton:@"OK"
							   alternateButton:NULL
								   otherButton:NULL
					 informativeTextWithFormat:@"I really appreciate your support. Enjoy Ethernal!"] runModal];
				[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithBool:YES] forKey:@"donated"];
				break;
			case NSAlertAlternateReturn:
				[self donate:self];
				break;
			case NSAlertOtherReturn:
				break;
				
		}		
	}
}

- (void)applicationWillTerminate:(NSNotification *)notif
{
	// Close all existing sockets
	NSEnumerator *enumerator = [[mSocketDictionary allValues] objectEnumerator];
	NSNumber *socket;
	while(socket = [enumerator nextObject]) {
		close([socket intValue]);			
	}
}

- (int)getSocketForSlot:(NSString*)slot
{
	NSNumber *socket = [mSocketDictionary objectForKey:slot];
	if(socket == NULL) {
		int bpfSocket = [PriviledgedSocketCreator createSocketAtSlot:slot];
		if(bpfSocket == -1)
			return -1;
		socket = [NSNumber numberWithInt:bpfSocket];
		[mSocketDictionary setObject:socket forKey:slot];
	}	
	return [socket intValue];
}

- (IBAction)donate:(id)sender
{
	[[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString:@"https://www.paypal.com/cgi-bin/webscr?cmd=_xclick&business=jean_bovet%40mac%2ecom&item_name=Ethernal%20Donation&item_number=17871&no_shipping=0&no_note=1&tax=0&currency_code=USD"]];
}

@end
