/*
 [The "BSD licence"]
 Copyright (c) 2005 Jean Bovet, http://www.cs.usfca.edu/~jbovet/
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
 modification, are permitted provided that the following conditions
 are met:
 
 1. Redistributions of source code must retain the above copyright
 notice, this list of conditions and the following disclaimer.
 2. Redistributions in binary form must reproduce the above copyright
 notice, this list of conditions and the following disclaimer in the
 documentation and/or other materials provided with the distribution.
 3. The name of the author may not be used to endorse or promote products
 derived from this software without specific prior written permission.
 
 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
														   NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
														   DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#import "Preferences.h"

@implementation Preferences

+ (NSMutableDictionary*)registerProtocol:(unsigned)protocol name:(NSString*)name sourceOffset:(unsigned)soffset destinationOffset:(unsigned)doffset
{
	NSMutableDictionary *dic = [NSMutableDictionary dictionary];
	[dic setObject:[NSString stringWithFormat:@"0x%04X", protocol] forKey:@"value"];
	[dic setObject:name forKey:@"name"];
	[dic setObject:[NSString stringWithFormat:@"%d", soffset] forKey:@"sourceIPOffset"];
	[dic setObject:[NSString stringWithFormat:@"%d", doffset] forKey:@"destinationIPOffset"];

	return dic;
}

+ (void)initialize
{
	if([[NSUserDefaults standardUserDefaults] objectForKey:@"protocols"] == NULL) {
		NSMutableArray *array = [NSMutableArray array];
		[array addObject:[Preferences registerProtocol:0x0800 name:@"IP" sourceOffset:26 destinationOffset:30]];
		[array addObject:[Preferences registerProtocol:0x0806 name:@"ARP" sourceOffset:28 destinationOffset:38]];
		[[NSUserDefaults standardUserDefaults] setObject:array forKey:@"protocols"];
	}
	if([[NSUserDefaults standardUserDefaults] objectForKey:@"ipAddressRepresentation"] == NULL)
		[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithInt:1] forKey:@"ipAddressRepresentation"];
	if([[NSUserDefaults standardUserDefaults] objectForKey:@"protocolRepresentation"] == NULL)
		[[NSUserDefaults standardUserDefaults] setObject:[NSNumber numberWithInt:1] forKey:@"protocolRepresentation"];
}

- (IBAction)addProtocol:(id)sender
{
	id o = [mProtocolController newObject];
	[mProtocolController addObject:o];
	[mProtocolController setSelectedObjects:[NSArray arrayWithObject:o]];
}

+ (unsigned)getIPOffsetForType:(unsigned)type key:(NSString*)key
{
	NSEnumerator *enumerator = [[[NSUserDefaults standardUserDefaults] objectForKey:@"protocols"] objectEnumerator];
	NSDictionary *dic;
	while(dic = [enumerator nextObject]) {
		NSScanner *scanner = [NSScanner scannerWithString:[dic objectForKey:@"value"]];
		unsigned value = 0;
		if([scanner scanHexInt:&value]) {
			if(value == type)
				return [[dic objectForKey:key] intValue];			
		}
	}
	return 0;
}

+ (BOOL)representIPAddressByValue
{
	return [[[NSUserDefaults standardUserDefaults] objectForKey:@"ipAddressRepresentation"] intValue] == 0;
}

+ (BOOL)representProtocolByValue
{
	return [[[NSUserDefaults standardUserDefaults] objectForKey:@"protocolRepresentation"] intValue] == 0;
}

+ (unsigned)sourceIPOffsetForType:(unsigned)type
{
	return [Preferences getIPOffsetForType:type key:@"sourceIPOffset"];
}

+ (unsigned)destinationIPOffsetForType:(unsigned)type
{
	return [Preferences getIPOffsetForType:type key:@"destinationIPOffset"];
}

+ (NSString*)nameOfProtocolType:(unsigned)type
{
	NSEnumerator *enumerator = [[[NSUserDefaults standardUserDefaults] objectForKey:@"protocols"] objectEnumerator];
	NSDictionary *dic;
	while(dic = [enumerator nextObject]) {
		NSScanner *scanner = [NSScanner scannerWithString:[dic objectForKey:@"value"]];
		unsigned value = 0;
		if([scanner scanHexInt:&value]) {
			if(value == type)
				return [dic objectForKey:@"name"];			
		}
	}
	return @"*";	
}

#include <netdb.h>

+ (NSString*)hostNameForIPAddress:(unsigned char*)addr
{
	static NSMutableDictionary *cache = NULL;
	if(cache == NULL)
		cache = [[NSMutableDictionary alloc] init];
	
	NSString *key = [NSString stringWithFormat:@"%08X", addr];
	NSString *name = [cache objectForKey:key];
	if(!name) {
		struct hostent *h = gethostbyaddr(addr, 4, AF_INET);
		if(h == NULL)
			name = [NSString stringWithFormat:@"%d.%d.%d.%d", addr[0], addr[1], addr[2], addr[3]];
		else
			name = [NSString stringWithFormat:@"%s", h->h_name];
		
		[cache setObject:name forKey:key];		
	}
		
	return name;
}

@end
